.TH RNAFOLD l
.ER
.SH NAME
RNAfold \- calculate secondary structures of RNAs
.SH SYNOPSIS
\fBRNAfold [\-p[\fI0\fP]] [\-C] [\-T\ \fItemp\fP] [\-4] [\-d[0|1|2|3]] [\-noLP] [\-noGU] [\-noCloseGU] [\-e\ \fI1|2\fP] [\-P\ \fIparamfile\fP] [\-nsp\ \fIpairs\fP] [\-S\ \fIscale\fP]

.SH DESCRIPTION
.I RNAfold
reads RNA sequences from stdin and calculates their minimum free energy
(mfe) structure, partition function (pf) and base pairing probability
matrix. It returns the mfe structure in bracket notation, its energy, the
free energy of the thermodynamic ensemble and the frequency of the mfe
structure in the ensemble to stdout.  It also produces PostScript files
with plots of the resulting secondary structure graph and a "dot plot" of
the base pairing matrix.  The dot plot shows a matrix of squares with area
proportional to the pairing probability in the upper half, and one square
for each pair in the minimum free energy structure in the lower half. For
each pair i-j with probability p>10E-6 there is a line of the form
.br
i  j  sqrt(p)  ubox
.br
in the PostScript file, so that the pair probabilities can be easily
extracted. 
If the sequence is preceded by a line of the form
.br
> name
.br
then the PostScript files name_ss.ps and name_dp.ps are produced for
the structure and dot plot, respectively. Otherwise the filenames
default to rna.ps and dot.ps. Existing files of the same name will be
overwritten.
.br
The program will continue to read new sequences until a line consisting
of the single character @ or an end of file condition is encountered.
.SH OPTIONS
.TP
.B \-p
Calculate the partition function and base pairing probability matrix in
addition to the mfe structure. Default is calculation of mfe structure only.
.TP
.B \-p0
Calculate the pf without pairing matrix and print the ensemble free energy
-kT ln(Z). This is much faster.
.TP
.B \-C
Calculate structures subject to constraints. 
The programm reads first the sequence then the a string containg constraints 
on the structure encoded with the symbols:  
. (no constraint for this base)
| (the corresponding base has to be paired
x (the base is unpaired)
< (base i is paired with a base j>i)
> (base i is paired with a base j<i)
and matching brackets ( ) (base i pairs base j)
With the exception of "|", constraints will disallow all pairs conflicting
with the constraint. This is usually sufficient to enforce the constraint,
but occasionally a base may stay unpaired in spite of constraints. PF
folding ignores constraints of type "|".
.TP
.B \-T\ \fItemp\fP
Rescale energy parameters to a temperature of \fItemp\fP C. Default is 37C.
.TP
.B \-4
Do not include special stabilizing energies for certain tetraloops. Mostly
for tesing.
.TP
.B \-d[0|1|2|3]
How to treat "dangling end" energies for bases adjacent to helices in free
ends and multiloops: Normally (-d1) only unpaired bases can participate in
at most one dangling end. With \fB-d2\fP this check is ignored, this is the
default for partition function folding (-p). \fB-d\fP or \FB-d0\fP ignores
dangling ends altogether. Note that by default pf and mfe folding treat
dangling ends differently, use -d2 in addition to -p to ensure that
both algorithms use the same energy model.
.br
With \fB-d3\fP mfe folding will allow coaxial stacking of adjacent helices
in multi-loops. At the moment the implementation will not allow coaxial
stacking of the two interior pairs in a loop of degree 3. PF folding will
work as with \fB-d2\fP.
.TP
.B \-noLP
Produce structures without lonely pairs (helices of length 1). 
For partition function folding this only disallows pairs that can 
\fBonly\fP occur isolated. Other pairs may still occasionally 
occur as helices of length 1.
.TP
.B \-noGU
Do not allow GU pairs.
.TP
.B \-noCloseGU
Do not allow GU pairs at the end of helices.
.TP
.B \-e\ \fI1|2\fP
Rarely used option to fold sequences from the artificial ABCD... alphabet,
where A pairs B, C-D etc.  Use the energy parameters for GC (-e 1) or AU
(-e 2) pairs.
.TP
.B \-P\ \fIparamfile\fP
Read energy parameters from \fIparamfile\fP, instead of using the default
parameter set. A sample parameterfile should accompany your distribution.
See the RNAlib documentation for details on the file format.
.TP
.B \-nsp  \fIpairs\fP
Allow other pairs in addition to the usual AU,GC,and GU pairs. \fIpairs\fP 
is a comma seperated list of additionally allowed pairs. If a the first 
character is a '-' then AB will imply that AB and BA are allowed pairs.
e.g. RNAfold -nsp -GA  will allow GA and AG pairs. Nonstandard pairs are 
given 0 stacking energy.
.TP
.B \-S\ \fIscale\fP
In the calculation of the pf use \fIscale\fP*mfe as an estimate for the
ensemble free energy (used to avoid overflows). The default is 1.07,
usefull values are 1.0 to 1.2. Occasionally needed for long sequences.
You can also recompile the programm to use double precision (see the README
file). 

.SH REFERENCES
The calculation of mfe structures is based on dynamic
programming algorithm originally developed by M. Zuker and P. Stiegler.
The partition function algorithm is based on work by J.S. McCaskill.
The energy parameters are taken from:
.br
D.H. Mathews, J. Sabina, M. Zucker and H. Turner
"Expanded Sequence Dependence of Thermodynamic Parameters Provides 
Robust Prediction of RNA Secondary Structure"
JMB, 288, pp 911-940, 1999
.br
A. Walter, D Turner, J Kim, M Lyttle, P Mller, D Mathews, M Zuker
"Coaxial stacking of helices enhances binding of oligoribonucleotides.."
PNAS, 91, pp 9218-9222, 1994
.PP
If you use this program in your work you might want to cite:
.PP
I.L. Hofacker, W. Fontana, P.F. Stadler, S. Bonhoeffer, M. Tacker, P. Schuster 
(1994)
Fast Folding and Comparison of RNA Secondary Structures.
Monatshefte f. Chemie 125: 167-188
.br
M. Zuker, P. Stiegler (1981) Optimal computer folding of large RNA
sequences using thermodynamic and auxiliary information, Nucl Acid Res
9: 133-148
.br
J.S. McCaskill (1990) The equilibrium partition function and base pair
binding probabilities for RNA secondary structures, Biopolymers 29: 1105-1119
.br
D. Adams (1979) The hitchhiker's guide to the galaxy, Pan Books, London
.SH VERSION
This man page documents version 1.4 Vienna RNA Package.
.SH AUTHORS
Ivo L Hofacker, Walter Fontana, Sebastian Bonhoeffer, Peter F Stadler.
.SH BUGS
If in doubt our program is right, nature is at fault.
Comments should be sent to ivo@tbi.univie.ac.at.
