.TH RNAINVERSE l
.ER
.SH NAME
RNAINVERSE \- find RNA sequences with given secondary structure
.SH SYNOPSIS
.B RNAinverse [\-F[mp]] [\-R \fIrepeats\fB]] [\-v] [\-a \fIalphabet\fB]
.B [\-f \fIfinal\fB] [\-T \fItemp\fB] [\-d[0|1|2|3] [\-4] [\-noGU] [\-noCloseGU]
.B [\-P \fIparamfile\fB] [\-e \fI1|2\fP] [\-nsp \fIpairs\fP]
.SH DESCRIPTION
.I RNAinverse
searches for sequences folding into a predefined structure, thereby
inverting the folding algorithm. Target structures (in bracket notation)
and starting sequences for the search are read alternately from stdin.
Characters in the start sequence other than 'AUGC' (or the alphabet
specified with \fB\-a\fP) will be treated as wild cards and replaced by a
random character. Any lower case characters in the start sequence will be
kept fixed during the search. If necessary, the sequence will be elongated
to the length of the structure. Thus a string of 'N's as well as a blank
line specify a random start sequence.
.br
For each search the best sequence found and its Hamming distance
to the start sequence are printed to stdout. If the the search was
unsuccessful, a structure distance to the target is appended.
The \fB\-Fp\fP and \fB\-R\fP options can modify the output format, see below.
.br
The program will continue to read new structures and sequences until a
line consisting of the single character @ or an end of file condition
is encountered. 
.SH OPTIONS
.TP
.B \-F[mp]
Use minimum energy (\fB\-Fm\fP), partition function folding (\fB\-Fp\fP)
or both (\fB\-Fmp\fP). 
In partition function mode, the probability of the target structure
exp\fI(\-E(S)/kT)/Q\fP is maximized. This probability is written in brackets
after the found sequence and Hamming distance. In most cases you'll want to
use the \fB\-f\fP option in conjunction with \fB\-Fp\fP, see below. The
default is \fB\-Fm\fP. 
.TP
.B \-f \fIfinal\fP
In combination with \fB\-Fp\fP stop search when sequence is found with
\fIE(s)\-F\fP is smaller than \fIfinal\fP, where \fIF=\-kT*ln(Q)\fP. 
.TP
.B \-R \fIrepeats\fP
Search repeatedly for the same structure. If \fIrepeats\fP is negative
search until \fI\-repeats\fP exact solutions are found, no output is done
for unsuccessful searches. Be aware, that the program will not terminate if
the target structure can not be found. 
.TP
.B \-v
in conjunction with \fB\-R\fP and a negative \fIrepeats\fP, print the
last subsequence and substructure for each unsuccessful search.
.TP
.B \-a \fIalphabet\fP
Find sequences using only bases from \fIalphabet\fP.
.PP
The \fB\-T\fP, \fB\-d\fP, \fB\-4\fP, \fB\-noGU\fP, \fB\-noCloseGU\fP,
\fB\-e\fP, \fB\-P\fP, \fB\-nsp\fP, options work as in RNAfold
.SH EXAMPLE
To search 5 times for sequences forming a simple hairpin structure
interrupted by one GA mismatch call
.br
.ft C
RNAinverse \-R 5
.ft
.br
and enter the lines
.br
.ft C
(((.(((....))).)))
.br
NNNgNNNNNNNNNNaNNN
.ft

.SH REFERENCES
The calculation of minimum free energy structures is based on dynamic
programming algorithm originally developed by M. Zuker and P. Stiegler.
The partition function algorithm is based on work by J.S. McCaskill.
.br
If this Program proves useful for your work please cite:
.PP
I.L. Hofacker, W. Fontana, P.F. Stadler, S. Bonhoeffer, M. Tacker, P. Schuster 
(1994)
Fast Folding and Comparison of RNA Secondary Structures.
Monatshefte f. Chemie 125: 167-188
.br
Other useful references are:
.br
M. Zuker, P. Stiegler (1981) Optimal computer folding of large RNA
sequences using thermodynamic and auxiliary information, Nucl Acid Res
9: 133-148
.br
J.S. McCaskill (1990) The equilibrium partition function and base pair
binding probabilities for RNA secondary structures, Biopolymers 29: 1105-1119
.br
D.H. Turner N. Sugimoto and S.M. Freier (1988) RNA structure
prediction, Ann Rev Biophys Biophys Chem 17: 167-192
.br
D. Adams (1979) The hitchhiker's guide to the galaxy, Pan Books, London
.SH VERSION
This man page documents version 1.4 Vienna RNA Package.
.SH AUTHOR
Ivo L Hofacker.
.SH BUGS
If in doubt our program is right, nature is at fault.
Comments should be sent to ivo@tbi.univie.ac.at.
.br
